@@ -1,11 +1,34 @@
-"""SCons.Action
+# MIT License
+#
+# Copyright The SCons Foundation
+#
+# Permission is hereby granted, free of charge, to any person obtaining
+# a copy of this software and associated documentation files (the
+# "Software"), to deal in the Software without restriction, including
+# without limitation the rights to use, copy, modify, merge, publish,
+# distribute, sublicense, and/or sell copies of the Software, and to
+# permit persons to whom the Software is furnished to do so, subject to
+# the following conditions:
+#
+# The above copyright notice and this permission notice shall be included
+# in all copies or substantial portions of the Software.
+#
+# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY
+# KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
+# WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
+# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
+# LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
+# OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
+# WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
-This encapsulates information about executing any sort of action that
+"""SCons Actions.
+
+Information about executing any sort of action that
 can build one or more target Nodes (typically files) from one or more
 source Nodes (also typically files) given a specific Environment.
 
 The base class here is ActionBase.  The base class supplies just a few
-OO utility methods and some generic methods for displaying information
+utility methods and some generic methods for displaying information
 about an Action in response to the various commands that control printing.
 
 A second-level base class is _ActionAction.  This extends ActionBase
@@ -60,7 +83,7 @@ this module:
     get_presig()
         Fetches the "contents" of a subclass for signature calculation.
         The varlist is added to this to produce the Action's contents.
-        TODO(?): Change this to always return ascii/bytes and not unicode (or py3 strings)
+        TODO(?): Change this to always return bytes and not str?
 
     strfunction()
         Returns a substituted string representation of the Action.
@@ -77,34 +100,12 @@ way for wrapping up the functions.
 
 """
 
-# Copyright (c) 2001 - 2019 The SCons Foundation
-#
-# Permission is hereby granted, free of charge, to any person obtaining
-# a copy of this software and associated documentation files (the
-# "Software"), to deal in the Software without restriction, including
-# without limitation the rights to use, copy, modify, merge, publish,
-# distribute, sublicense, and/or sell copies of the Software, and to
-# permit persons to whom the Software is furnished to do so, subject to
-# the following conditions:
-#
-# The above copyright notice and this permission notice shall be included
-# in all copies or substantial portions of the Software.
-#
-# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY
-# KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
-# WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
-# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
-# LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
-# OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
-# WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
-
-__revision__ = "/src/script/src/engine/SCons/Action.py none 2019-12-16 19:06:27 none"
-
 import os
 import pickle
 import re
 import sys
 import subprocess
+from subprocess import DEVNULL
 import itertools
 import inspect
 from collections import OrderedDict
@@ -118,7 +119,7 @@ import SCons.Subst
 # we use these a lot, so try to optimize them
 from SCons.Util import is_String, is_List
 
-class _null(object):
+class _null:
     pass
 
 print_actions = 1
@@ -514,7 +515,7 @@ def Action(act, *args, **kw):
     return _do_create_action(act, kw)
 
 
-class ActionBase(object):
+class ActionBase:
     """Base class for all types of action objects that can be held by
     other objects (Builders, Executors, etc.)  This provides the
     common methods for manipulating and combining those actions."""
@@ -628,17 +629,9 @@ class _ActionAction(ActionBase):
         """
         In python 3, and in some of our tests, sys.stdout is
         a String io object, and it takes unicode strings only
-        In other cases it's a regular Python 2.x file object
-        which takes strings (bytes), and if you pass those a
-        unicode object they try to decode with 'ascii' codec
-        which fails if the cmd line has any hi-bit-set chars.
-        This code assumes s is a regular string, but should
-        work if it's unicode too.
+        This code assumes s is a regular string.
         """
-        try:
-            sys.stdout.write(s + u"\n")
-        except UnicodeDecodeError:
-            sys.stdout.write(s + "\n")
+        sys.stdout.write(s + "\n")
 
     def __call__(self, target, source, env,
                                exitstatfunc=_null,
@@ -656,10 +649,14 @@ class _ActionAction(ActionBase):
             presub = self.presub
             if presub is _null:
                 presub = print_actions_presub
-        if exitstatfunc is _null: exitstatfunc = self.exitstatfunc
-        if show is _null:  show = print_actions
-        if execute is _null:  execute = execute_actions
-        if chdir is _null: chdir = self.chdir
+        if exitstatfunc is _null:
+            exitstatfunc = self.exitstatfunc
+        if show is _null:
+            show = print_actions
+        if execute is _null:
+            execute = execute_actions
+        if chdir is _null:
+            chdir = self.chdir
         save_cwd = None
         if chdir:
             save_cwd = os.getcwd()
@@ -677,7 +674,7 @@ class _ActionAction(ActionBase):
                 source = executor.get_all_sources()
             t = ' and '.join(map(str, target))
             l = '\n  '.join(self.presub_lines(env))
-            out = u"Building %s with action:\n  %s\n" % (t, l)
+            out = "Building %s with action:\n  %s\n" % (t, l)
             sys.stdout.write(out)
         cmd = None
         if show and self.strfunction:
@@ -759,30 +756,20 @@ def get_default_ENV(env):
         return default_ENV
 
 
-def _subproc(scons_env, cmd, error = 'ignore', **kw):
-    """Do common setup for a subprocess.Popen() call
+def _subproc(scons_env, cmd, error='ignore', **kw):
+    """Wrapper for subprocess which pulls from construction env.
 
-    This function is still in draft mode.  We're going to need something like
-    it in the long run as more and more places use subprocess, but I'm sure
-    it'll have to be tweaked to get the full desired functionality.
-    one special arg (so far?), 'error', to tell what to do with exceptions.
+    Use for calls to subprocess which need to interpolate values from
+    an SCons construction environment into the environment passed to
+    subprocess.  Adds an an error-handling argument.  Adds ability
+    to specify std{in,out,err} with "'devnull'" tag.
     """
-    # allow std{in,out,err} to be "'devnull'".  This is like
-    # subprocess.DEVNULL, which does not exist for Py2. Use the
-    # subprocess one if possible.
-    # Clean this up when Py2 support is dropped
-    try:
-        from subprocess import DEVNULL
-    except ImportError:
-        DEVNULL = None
-
+    # TODO: just uses subprocess.DEVNULL now, we can drop the "devnull"
+    # string now - it is a holdover from Py2, which didn't have DEVNULL.
     for stream in 'stdin', 'stdout', 'stderr':
         io = kw.get(stream)
         if is_String(io) and io == 'devnull':
-            if DEVNULL:
-                kw[stream] = DEVNULL
-            else:
-                kw[stream] = open(os.devnull, "r+")
+            kw[stream] = DEVNULL
 
     # Figure out what shell environment to use
     ENV = kw.get('env', None)
@@ -812,12 +799,12 @@ def _subproc(scons_env, cmd, error = 'ignore', **kw):
     except EnvironmentError as e:
         if error == 'raise': raise
         # return a dummy Popen instance that only returns error
-        class dummyPopen(object):
+        class dummyPopen:
             def __init__(self, e): self.exception = e
             def communicate(self, input=None): return ('', '')
             def wait(self): return -self.exception.errno
             stdin = None
-            class f(object):
+            class f:
                 def read(self): return ''
                 def readline(self): return ''
                 def __iter__(self): return iter(())
@@ -976,11 +963,33 @@ class CommandAction(_ActionAction):
             return env.subst_target_source(cmd, SUBST_SIG, target, source)
 
     def get_implicit_deps(self, target, source, env, executor=None):
+        """Return the implicit dependencies of this action's command line."""
         icd = env.get('IMPLICIT_COMMAND_DEPENDENCIES', True)
         if is_String(icd) and icd[:1] == '$':
             icd = env.subst(icd)
-        if not icd or icd in ('0', 'None'):
+
+        if not icd or str(icd).lower in ('0', 'none', 'false', 'no', 'off'):
             return []
+
+        try:
+            icd_int = int(icd)
+        except ValueError:
+            icd_int = None
+
+        if (icd_int and icd_int > 1) or icd == 'all':
+            # An integer value greater than 1 specifies the number of entries
+            # to scan. "all" means to scan all.
+            return self._get_implicit_deps_heavyweight(target, source, env, executor, icd_int)
+        else:
+            # Everything else (usually 1 or True) means that we want
+            # lightweight dependency scanning.
+            return self._get_implicit_deps_lightweight(target, source, env, executor)
+
+    def _get_implicit_deps_lightweight(self, target, source, env, executor):
+        """
+        Lightweight dependency scanning involves only scanning the first entry
+        in an action string, even if it contains &&.
+        """
         from SCons.Subst import SUBST_SIG
         if executor:
             cmd_list = env.subst_list(self.cmd_list, SUBST_SIG, executor=executor)
@@ -998,6 +1007,65 @@ class CommandAction(_ActionAction):
                     res.append(env.fs.File(d))
         return res
 
+    def _get_implicit_deps_heavyweight(self, target, source, env, executor,
+                                       icd_int):
+        """
+        Heavyweight dependency scanning involves scanning more than just the
+        first entry in an action string. The exact behavior depends on the
+        value of icd_int. Only files are taken as implicit dependencies;
+        directories are ignored.
+
+        If icd_int is an integer value, it specifies the number of entries to
+        scan for implicit dependencies. Action strings are also scanned after
+        a &&. So for example, if icd_int=2 and the action string is
+        "cd <some_dir> && $PYTHON $SCRIPT_PATH <another_path>", the implicit
+        dependencies would be the path to the python binary and the path to the
+        script.
+
+        If icd_int is None, all entries are scanned for implicit dependencies.
+        """
+
+        # Avoid circular and duplicate dependencies by not providing source,
+        # target, or executor to subst_list. This causes references to
+        # $SOURCES, $TARGETS, and all related variables to disappear.
+        from SCons.Subst import SUBST_SIG
+        cmd_list = env.subst_list(self.cmd_list, SUBST_SIG, conv=lambda x: x)
+        res = []
+
+        for cmd_line in cmd_list:
+            if cmd_line:
+                entry_count = 0
+                for entry in cmd_line:
+                    d = str(entry)
+                    if ((icd_int is None or entry_count < icd_int) and
+                            not d.startswith(('&', '-', '/') if os.name == 'nt'
+                                             else ('&', '-'))):
+                        m = strip_quotes.match(d)
+                        if m:
+                            d = m.group(1)
+
+                        if d:
+                            # Resolve the first entry in the command string using
+                            # PATH, which env.WhereIs() looks in.
+                            # For now, only match files, not directories.
+                            p = os.path.abspath(d) if os.path.isfile(d) else None
+                            if not p and entry_count == 0:
+                                p = env.WhereIs(d)
+
+                            if p:
+                                res.append(env.fs.File(p))
+
+                        entry_count = entry_count + 1
+                    else:
+                        entry_count = 0 if d == '&&' else entry_count + 1
+
+        # Despite not providing source and target to env.subst() above, we
+        # can still end up with sources in this list. For example, files in
+        # LIBS will still resolve in env.subst(). This won't result in
+        # circular dependencies, but it causes problems with cache signatures
+        # changing between full and incremental builds.
+        return [r for r in res if r not in target and r not in source]
+
 
 class CommandGeneratorAction(ActionBase):
     """Class for command-generator actions."""
@@ -1046,11 +1114,14 @@ class CommandGeneratorAction(ActionBase):
                  show=_null, execute=_null, chdir=_null, executor=None):
         act = self._generate(target, source, env, 0, executor)
         if act is None:
-            raise SCons.Errors.UserError("While building `%s': "
-                            "Cannot deduce file extension from source files: %s"
-                % (repr(list(map(str, target))), repr(list(map(str, source)))))
-        return act(target, source, env, exitstatfunc, presub,
-                   show, execute, chdir, executor)
+            raise SCons.Errors.UserError(
+                "While building `%s': "
+                "Cannot deduce file extension from source files: %s"
+                % (repr(list(map(str, target))), repr(list(map(str, source))))
+            )
+        return act(
+            target, source, env, exitstatfunc, presub, show, execute, chdir, executor
+        )
 
     def get_presig(self, target, source, env, executor=None):
         """Return the signature contents of this action's command line.
@@ -1308,7 +1379,7 @@ class ListAction(ActionBase):
         return list(result.keys())
 
 
-class ActionCaller(object):
+class ActionCaller:
     """A class for delaying calling an Action function with specific
     (positional and keyword) arguments until the Action is actually
     executed.
@@ -1379,7 +1450,7 @@ class ActionCaller(object):
         return self.parent.strfunc(*self.args, **self.kw)
 
 
-class ActionFactory(object):
+class ActionFactory:
     """A factory class that will wrap up an arbitrary function
     as an SCons-executable Action object.
 
