@@ -1,5 +1,6 @@
+# MIT License
 #
-# Copyright (c) 2001 - 2019 The SCons Foundation
+# Copyright The SCons Foundation
 #
 # Permission is hereby granted, free of charge, to any person obtaining
 # a copy of this software and associated documentation files (the
@@ -19,24 +20,19 @@
 # LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 # OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 # WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
-#
 
-__revision__ = "/src/script/src/engine/SCons/CacheDir.py none 2019-12-16 19:06:27 none"
-
-__doc__ = """
-CacheDir support
+"""CacheDir support
 """
 
-import hashlib
+import atexit
 import json
 import os
 import stat
 import sys
 
-import SCons
 import SCons.Action
+import SCons.Errors
 import SCons.Warnings
-from SCons.Util import PY3
 
 cache_enabled = True
 cache_debug = False
@@ -136,11 +132,8 @@ def CachePushFunc(target, source, env):
 
 CachePush = SCons.Action.Action(CachePushFunc, None)
 
-# Nasty hack to cut down to one warning for each cachedir path that needs
-# upgrading.
-warned = dict()
 
-class CacheDir(object):
+class CacheDir:
 
     def __init__(self, path):
         """
@@ -160,15 +153,12 @@ class CacheDir(object):
         if path is None:
             return
 
-        if PY3:
-            self._readconfig3(path)
-        else:
-            self._readconfig2(path)
+        self._readconfig(path)
 
 
-    def _readconfig3(self, path):
+    def _readconfig(self, path):
         """
-        Python3 version of reading the cache config.
+        Read the cache config.
 
         If directory or config file do not exist, create.  Take advantage
         of Py3 capability in os.makedirs() and in file open(): just try
@@ -204,71 +194,16 @@ class CacheDir(object):
                 msg = "Failed to read cache configuration for " + path
                 raise SCons.Errors.SConsEnvironmentError(msg)
 
-
-    def _readconfig2(self, path):
-        """
-        Python2 version of reading cache config.
-
-        See if there is a config file in the cache directory. If there is,
-        use it. If there isn't, and the directory exists and isn't empty,
-        produce a warning. If the directory does not exist or is empty,
-        write a config file.
-
-        :param path: path to the cache directory
-        """
-        config_file = os.path.join(path, 'config')
-        if not os.path.exists(config_file):
-            # A note: There is a race hazard here if two processes start and
-            # attempt to create the cache directory at the same time. However,
-            # Python 2.x does not give you the option to do exclusive file
-            # creation (not even the option to error on opening an existing
-            # file for writing...). The ordering of events here is an attempt
-            # to alleviate this, on the basis that it's a pretty unlikely
-            # occurrence (would require two builds with a brand new cache
-            # directory)
-            if os.path.isdir(path) and any(f != "config" for f in os.listdir(path)):
-                self.config['prefix_len'] = 1
-                # When building the project I was testing this on, the warning
-                # was output over 20 times. That seems excessive
-                global warned
-                if self.path not in warned:
-                    msg = "Please upgrade your cache by running " +\
-                          "scons-configure-cache.py " +  self.path
-                    SCons.Warnings.warn(SCons.Warnings.CacheVersionWarning, msg)
-                    warned[self.path] = True
-            else:
-                if not os.path.isdir(path):
-                    try:
-                        os.makedirs(path)
-                    except OSError:
-                        # If someone else is trying to create the directory at
-                        # the same time as me, bad things will happen
-                        msg = "Failed to create cache directory " + path
-                        raise SCons.Errors.SConsEnvironmentError(msg)
-
-                self.config['prefix_len'] = 2
-                if not os.path.exists(config_file):
-                    try:
-                        with open(config_file, 'w') as config:
-                            json.dump(self.config, config)
-                    except Exception:
-                        msg = "Failed to write cache configuration for " + path
-                        raise SCons.Errors.SConsEnvironmentError(msg)
-        else:
-            try:
-                with open(config_file) as config:
-                    self.config = json.load(config)
-            except ValueError:
-                msg = "Failed to read cache configuration for " + path
-                raise SCons.Errors.SConsEnvironmentError(msg)
-
-
     def CacheDebug(self, fmt, target, cachefile):
         if cache_debug != self.current_cache_debug:
             if cache_debug == '-':
                 self.debugFP = sys.stdout
             elif cache_debug:
+                def debug_cleanup(debugFP):
+                    debugFP.close()
+
                 self.debugFP = open(cache_debug, 'w')
+                atexit.register(debug_cleanup, self.debugFP)
             else:
                 self.debugFP = None
             self.current_cache_debug = cache_debug
