@@ -1,15 +1,6 @@
-"""SCons.Debug
-
-Code for debugging SCons internal things.  Shouldn't be
-needed by most users. Quick shortcuts:
-
-from SCons.Debug import caller_trace
-caller_trace()
-
-"""
-
+# MIT License
 #
-# Copyright (c) 2001 - 2019 The SCons Foundation
+# Copyright The SCons Foundation
 #
 # Permission is hereby granted, free of charge, to any person obtaining
 # a copy of this software and associated documentation files (the
@@ -29,10 +20,16 @@ caller_trace()
 # LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 # OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 # WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
-#
 
-__revision__ = "/src/script/src/engine/SCons/Debug.py none 2019-12-17 02:06:27 none"
+"""Code for debugging SCons internal things.
 
+Shouldn't be needed by most users. Quick shortcuts:
+
+from SCons.Debug import caller_trace
+caller_trace()
+"""
+
+import atexit
 import os
 import sys
 import time
@@ -66,7 +63,7 @@ def string_to_classes(s):
 def fetchLoggedInstances(classes="*"):
     classnames = string_to_classes(classes)
     return [(cn, len(tracked_classes[cn])) for cn in classnames]
-  
+
 def countLoggedInstances(classes, file=sys.stdout):
     for classname in string_to_classes(classes):
         file.write("%s: %d\n" % (classname, len(tracked_classes[classname])))
@@ -93,7 +90,6 @@ def dumpLoggedInstances(classes, file=sys.stdout):
                     file.write('        %20s : %s\n' % (key, value))
 
 
-
 if sys.platform[:5] == "linux":
     # Linux doesn't actually support memory usage stats from getrusage().
     def memory():
@@ -105,28 +101,23 @@ elif sys.platform[:6] == 'darwin':
     #TODO really get memory stats for OS X
     def memory():
         return 0
+elif sys.platform == 'win32':
+    from SCons.compat.win32 import get_peak_memory_usage
+    memory = get_peak_memory_usage
 else:
     try:
         import resource
     except ImportError:
-        try:
-            import win32process
-            import win32api
-        except ImportError:
-            def memory():
-                return 0
-        else:
-            def memory():
-                process_handle = win32api.GetCurrentProcess()
-                memory_info = win32process.GetProcessMemoryInfo( process_handle )
-                return memory_info['PeakWorkingSetSize']
+        def memory():
+            return 0
     else:
         def memory():
             res = resource.getrusage(resource.RUSAGE_SELF)
             return res[4]
 
-# returns caller's stack
+
 def caller_stack():
+    """return caller's stack"""
     import traceback
     tb = traceback.extract_stack()
     # strip itself and the caller from the output
@@ -201,40 +192,57 @@ if sys.platform == 'win32':
     TraceDefault = 'con'
 else:
     TraceDefault = '/dev/tty'
-
-TimeStampDefault = None
+TimeStampDefault = False
 StartTime = time.time()
 PreviousTime = StartTime
 
-def Trace(msg, file=None, mode='w', tstamp=None):
-    """Write a trace message to a file.  Whenever a file is specified,
-    it becomes the default for the next call to Trace()."""
+def Trace(msg, tracefile=None, mode='w', tstamp=False):
+    """Write a trace message.
+
+    Write messages when debugging which do not interfere with stdout.
+    Useful in tests, which monitor stdout and would break with
+    unexpected output. Trace messages can go to the console (which is
+    opened as a file), or to a disk file; the tracefile argument persists
+    across calls unless overridden.
+
+    Args:
+        tracefile: file to write trace message to. If omitted,
+          write to the previous trace file (default: console).
+        mode: file open mode (default: 'w')
+        tstamp: write relative timestamps with trace. Outputs time since
+          scons was started, and time since last trace (default: False)
+
+    """
     global TraceDefault
     global TimeStampDefault
     global PreviousTime
-    if file is None:
-        file = TraceDefault
+
+    def trace_cleanup(traceFP):
+        traceFP.close()
+
+    if tracefile is None:
+        tracefile = TraceDefault
     else:
-        TraceDefault = file
-    if tstamp is None:
+        TraceDefault = tracefile
+    if not tstamp:
         tstamp = TimeStampDefault
     else:
         TimeStampDefault = tstamp
     try:
-        fp = TraceFP[file]
+        fp = TraceFP[tracefile]
     except KeyError:
         try:
-            fp = TraceFP[file] = open(file, mode)
+            fp = TraceFP[tracefile] = open(tracefile, mode)
+            atexit.register(trace_cleanup, fp)
         except TypeError:
             # Assume we were passed an open file pointer.
-            fp = file
+            fp = tracefile
     if tstamp:
         now = time.time()
         fp.write('%8.4f %8.4f:  ' % (now - StartTime, now - PreviousTime))
         PreviousTime = now
     fp.write(msg)
     fp.flush()
-    fp.close()
 
 # Local Variables:
 # tab-width:4
