@@ -1,5 +1,6 @@
+# MIT License
 #
-# Copyright (c) 2001 - 2019 The SCons Foundation
+# Copyright The SCons Foundation
 #
 # Permission is hereby granted, free of charge, to any person obtaining
 # a copy of this software and associated documentation files (the
@@ -20,46 +21,35 @@
 # OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 # WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
-from __future__ import print_function
+"""Generic Taskmaster module for the SCons build engine.
 
-import sys
+This module contains the primary interface(s) between a wrapping user
+interface and the SCons build engine.  There are two key classes here:
 
-__doc__ = """
-    Generic Taskmaster module for the SCons build engine.
-    =====================================================
+Taskmaster
+    This is the main engine for walking the dependency graph and
+    calling things to decide what does or doesn't need to be built.
 
-    This module contains the primary interface(s) between a wrapping user
-    interface and the SCons build engine.  There are two key classes here:
+Task
+    This is the base class for allowing a wrapping interface to
+    decide what does or doesn't actually need to be done.  The
+    intention is for a wrapping interface to subclass this as
+    appropriate for different types of behavior it may need.
 
-    Taskmaster
-    ----------
-        This is the main engine for walking the dependency graph and
-        calling things to decide what does or doesn't need to be built.
+    The canonical example is the SCons native Python interface,
+    which has Task subclasses that handle its specific behavior,
+    like printing "'foo' is up to date" when a top-level target
+    doesn't need to be built, and handling the -c option by removing
+    targets as its "build" action.  There is also a separate subclass
+    for suppressing this output when the -q option is used.
 
-    Task
-    ----
-        This is the base class for allowing a wrapping interface to
-        decide what does or doesn't actually need to be done.  The
-        intention is for a wrapping interface to subclass this as
-        appropriate for different types of behavior it may need.
-
-        The canonical example is the SCons native Python interface,
-        which has Task subclasses that handle its specific behavior,
-        like printing "'foo' is up to date" when a top-level target
-        doesn't need to be built, and handling the -c option by removing
-        targets as its "build" action.  There is also a separate subclass
-        for suppressing this output when the -q option is used.
-
-        The Taskmaster instantiates a Task object for each (set of)
-        target(s) that it decides need to be evaluated and/or built.
+    The Taskmaster instantiates a Task object for each (set of)
+    target(s) that it decides need to be evaluated and/or built.
 """
 
-__revision__ = "/src/script/src/engine/SCons/Taskmaster.py none 2019-12-16 19:06:27 none"
-
-from itertools import chain
-import operator
 import sys
-import traceback
+from abc import ABC, abstractmethod
+from itertools import chain
 
 import SCons.Errors
 import SCons.Node
@@ -81,7 +71,7 @@ print_prepare = 0               # set by option --debug=prepare
 
 CollectStats = None
 
-class Stats(object):
+class Stats:
     """
     A simple class for holding statistics about the disposition of a
     Node by the Taskmaster.  If we're collecting statistics, each Node
@@ -117,10 +107,8 @@ def dump_stats():
         print((fmt % n.attributes.stats.__dict__) + str(n))
 
 
-
-class Task(object):
-    """
-    Default SCons build engine task.
+class Task(ABC):
+    """ SCons build engine abstract task class.
 
     This controls the interaction of the actual building of node
     and the rest of the engine.
@@ -171,7 +159,7 @@ class Task(object):
         """
         global print_prepare
         T = self.tm.trace
-        if T: T.write(self.trace_message(u'Task.prepare()', self.node))
+        if T: T.write(self.trace_message('Task.prepare()', self.node))
 
         # Now that it's the appropriate time, give the TaskMaster a
         # chance to raise any exceptions it encountered while preparing
@@ -212,17 +200,9 @@ class Task(object):
         """
         return self.node
 
+    @abstractmethod
     def needs_execute(self):
-        # TODO(deprecate):  "return True" is the old default behavior;
-        # change it to NotImplementedError (after running through the
-        # Deprecation Cycle) so the desired behavior is explicitly
-        # determined by which concrete subclass is used.
-        #raise NotImplementedError
-        msg = ('Taskmaster.Task is an abstract base class; instead of\n'
-              '\tusing it directly, '
-              'derive from it and override the abstract methods.')
-        SCons.Warnings.warn(SCons.Warnings.TaskmasterNeedsExecuteWarning, msg)
-        return True
+        return
 
     def execute(self):
         """
@@ -233,7 +213,7 @@ class Task(object):
         prepare(), executed() or failed().
         """
         T = self.tm.trace
-        if T: T.write(self.trace_message(u'Task.execute()', self.node))
+        if T: T.write(self.trace_message('Task.execute()', self.node))
 
         try:
             cached_targets = []
@@ -399,7 +379,7 @@ class Task(object):
         """
         global print_prepare
         T = self.tm.trace
-        if T: T.write(self.trace_message(u'Task.make_ready_current()',
+        if T: T.write(self.trace_message('Task.make_ready_current()',
                                          self.node))
 
         self.out_of_date = []
@@ -447,7 +427,7 @@ class Task(object):
         that can be put back on the candidates list.
         """
         T = self.tm.trace
-        if T: T.write(self.trace_message(u'Task.postprocess()', self.node))
+        if T: T.write(self.trace_message('Task.postprocess()', self.node))
 
         # We may have built multiple targets, some of which may have
         # common parents waiting for this build.  Count up how many
@@ -464,7 +444,7 @@ class Task(object):
             # A node can only be in the pending_children set if it has
             # some waiting_parents.
             if t.waiting_parents:
-                if T: T.write(self.trace_message(u'Task.postprocess()',
+                if T: T.write(self.trace_message('Task.postprocess()',
                                                  t,
                                                  'removing'))
                 pending_children.discard(t)
@@ -493,7 +473,7 @@ class Task(object):
 
         for p, subtract in parents.items():
             p.ref_count = p.ref_count - subtract
-            if T: T.write(self.trace_message(u'Task.postprocess()',
+            if T: T.write(self.trace_message('Task.postprocess()',
                                              p,
                                              'adjusted parent ref count'))
             if p.ref_count == 0:
@@ -555,15 +535,12 @@ class Task(object):
             exc_traceback = None
 
         # raise exc_type(exc_value).with_traceback(exc_traceback)
-        if sys.version_info[0] == 2:
-            exec("raise exc_type, exc_value, exc_traceback")
-        else: #  sys.version_info[0] == 3:
-            if isinstance(exc_value, Exception): #hasattr(exc_value, 'with_traceback'):
-                # If exc_value is an exception, then just reraise
-                exec("raise exc_value.with_traceback(exc_traceback)")
-            else:
-                # else we'll create an exception using the value and raise that
-                exec("raise exc_type(exc_value).with_traceback(exc_traceback)")
+        if isinstance(exc_value, Exception): #hasattr(exc_value, 'with_traceback'):
+            # If exc_value is an exception, then just reraise
+            raise exc_value.with_traceback(exc_traceback)
+        else:
+            # else we'll create an exception using the value and raise that
+            raise exc_type(exc_value).with_traceback(exc_traceback)
 
 
         # raise e.__class__, e.__class__(e), sys.exc_info()[2]
@@ -582,7 +559,7 @@ class AlwaysTask(Task):
         dependencies) can use this as follows:
 
             class MyTaskSubclass(SCons.Taskmaster.Task):
-                needs_execute = SCons.Taskmaster.Task.execute_always
+                needs_execute = SCons.Taskmaster.AlwaysTask.needs_execute
         """
         return True
 
@@ -610,7 +587,7 @@ def find_cycle(stack, visited):
     return None
 
 
-class Taskmaster(object):
+class Taskmaster:
     """
     The Taskmaster for walking the dependency DAG.
     """
@@ -792,12 +769,12 @@ class Taskmaster(object):
         self.ready_exc = None
 
         T = self.trace
-        if T: T.write(SCons.Util.UnicodeType('\n') + self.trace_message('Looking for a node to evaluate'))
+        if T: T.write('\n' + self.trace_message('Looking for a node to evaluate'))
 
         while True:
             node = self.next_candidate()
             if node is None:
-                if T: T.write(self.trace_message('No candidate anymore.') + u'\n')
+                if T: T.write(self.trace_message('No candidate anymore.') + '\n')
                 return None
 
             node = node.disambiguate()
@@ -820,7 +797,7 @@ class Taskmaster(object):
             else:
                 S = None
 
-            if T: T.write(self.trace_message(u'    Considering node %s and its children:' % self.trace_node(node)))
+            if T: T.write(self.trace_message('    Considering node %s and its children:' % self.trace_node(node)))
 
             if state == NODE_NO_STATE:
                 # Mark this node as being on the execution stack:
@@ -828,7 +805,7 @@ class Taskmaster(object):
             elif state > NODE_PENDING:
                 # Skip this node if it has already been evaluated:
                 if S: S.already_handled = S.already_handled + 1
-                if T: T.write(self.trace_message(u'       already handled (executed)'))
+                if T: T.write(self.trace_message('       already handled (executed)'))
                 continue
 
             executor = node.get_executor()
@@ -859,7 +836,7 @@ class Taskmaster(object):
             for child in chain(executor.get_all_prerequisites(), children):
                 childstate = child.get_state()
 
-                if T: T.write(self.trace_message(u'       ' + self.trace_node(child)))
+                if T: T.write(self.trace_message('       ' + self.trace_node(child)))
 
                 if childstate == NODE_NO_STATE:
                     children_not_visited.append(child)
@@ -920,7 +897,7 @@ class Taskmaster(object):
                     # count so we can be put back on the list for
                     # re-evaluation when they've all finished.
                     node.ref_count =  node.ref_count + child.add_to_waiting_parents(node)
-                    if T: T.write(self.trace_message(u'     adjusted ref count: %s, child %s' %
+                    if T: T.write(self.trace_message('     adjusted ref count: %s, child %s' %
                                   (self.trace_node(node), repr(str(child)))))
 
                 if T:
@@ -946,7 +923,7 @@ class Taskmaster(object):
             # The default when we've gotten through all of the checks above:
             # this node is ready to be built.
             if S: S.build = S.build + 1
-            if T: T.write(self.trace_message(u'Evaluating %s\n' %
+            if T: T.write(self.trace_message('Evaluating %s\n' %
                                              self.trace_node(node)))
 
             # For debugging only:
