@@ -3,7 +3,7 @@
 Used by several tools of `gettext` toolset.
 """
 
-# Copyright (c) 2001 - 2019 The SCons Foundation
+# __COPYRIGHT__
 #
 # Permission is hereby granted, free of charge, to any person obtaining
 # a copy of this software and associated documentation files (the
@@ -24,32 +24,32 @@ Used by several tools of `gettext` toolset.
 # OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 # WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
-__revision__ = "/src/script/src/engine/SCons/Tool/GettextCommon.py none 2019-12-17 02:06:27 none"
+__revision__ = "__FILE__ __REVISION__ __DATE__ __DEVELOPER__"
 
 import SCons.Warnings
 import re
 
 
 #############################################################################
-class XgettextToolWarning(SCons.Warnings.Warning): pass
+class XgettextToolWarning(SCons.Warnings.SConsWarning): pass
 
 
 class XgettextNotFound(XgettextToolWarning): pass
 
 
-class MsginitToolWarning(SCons.Warnings.Warning): pass
+class MsginitToolWarning(SCons.Warnings.SConsWarning): pass
 
 
 class MsginitNotFound(MsginitToolWarning): pass
 
 
-class MsgmergeToolWarning(SCons.Warnings.Warning): pass
+class MsgmergeToolWarning(SCons.Warnings.SConsWarning): pass
 
 
 class MsgmergeNotFound(MsgmergeToolWarning): pass
 
 
-class MsgfmtToolWarning(SCons.Warnings.Warning): pass
+class MsgfmtToolWarning(SCons.Warnings.SConsWarning): pass
 
 
 class MsgfmtNotFound(MsgfmtToolWarning): pass
@@ -69,7 +69,7 @@ SCons.Warnings.enableWarningClass(MsgfmtNotFound)
 #############################################################################
 
 #############################################################################
-class _POTargetFactory(object):
+class _POTargetFactory:
     """ A factory of `PO` target files.
 
     Factory defaults differ from these of `SCons.Node.FS.FS`.  We set `precious`
@@ -269,7 +269,7 @@ def _translate(env, target=None, source=SCons.Environment._null, *args, **kw):
 #############################################################################
 
 #############################################################################
-class RPaths(object):
+class RPaths:
     """ Callable object, which returns pathnames relative to SCons current
     working directory.
 
