@@ -10,7 +10,7 @@ selection method.
 """
 
 #
-# Copyright (c) 2001 - 2019 The SCons Foundation
+# __COPYRIGHT__
 #
 # Permission is hereby granted, free of charge, to any person obtaining
 # a copy of this software and associated documentation files (the
@@ -30,11 +30,13 @@ selection method.
 # LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 # OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 # WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
-from __future__ import division, print_function
+__revision__ = "__FILE__ __REVISION__ __DATE__ __DEVELOPER__"
 
-__revision__ = "/src/script/src/engine/SCons/Tool/intelc.py none 2019-12-16 19:06:27 none"
-
-import math, sys, os.path, glob, string, re
+import glob
+import math
+import os.path
+import re
+import sys
 
 is_windows = sys.platform == 'win32'
 is_win64 = is_windows and (os.environ['PROCESSOR_ARCHITECTURE'] == 'AMD64' or
@@ -318,7 +320,7 @@ def get_intel_compiler_top(version, abi):
         if not os.path.exists(os.path.join(top, "Bin", "icl.exe")) \
               and not os.path.exists(os.path.join(top, "Bin", abi, "icl.exe")) \
               and not os.path.exists(os.path.join(top, "Bin", archdir, "icl.exe")):
-            raise MissingDirError("Can't find Intel compiler in %s"%(top))
+            raise MissingDirError("Can't find Intel compiler in %s" % top)
     elif is_mac or is_linux:
         def find_in_2008style_dir(version):
             # first dir is new (>=9.0) style, second is old (8.0) style.
@@ -389,13 +391,15 @@ def get_intel_compiler_top(version, abi):
 def generate(env, version=None, abi=None, topdir=None, verbose=0):
     r"""Add Builders and construction variables for Intel C/C++ compiler
     to an Environment.
-    args:
-      version: (string) compiler version to use, like "80"
-      abi:     (string) 'win32' or whatever Itanium version wants
-      topdir:  (string) compiler top dir, like
-                         "c:\Program Files\Intel\Compiler70"
-                        If topdir is used, version and abi are ignored.
-      verbose: (int)    if >0, prints compiler version used.
+
+    Args:
+      version (str): compiler version to use, like "80"
+      abi (str): 'win32' or whatever Itanium version wants
+      topdir (str): directory containing compiler tree, e.g.
+        "c:\\Program Files\\Intel\\Compiler70".
+        If `topdir` is used, `version` and `abi` are ignored.
+      verbose: if >0, prints compiler version used.
+
     """
     if not (is_mac or is_linux or is_windows):
         # can't handle this platform
@@ -448,7 +452,7 @@ def generate(env, version=None, abi=None, topdir=None, verbose=0):
     if not topdir:
         # Normally this is an error, but it might not be if the compiler is
         # on $PATH and the user is importing their env.
-        class ICLTopDirWarning(SCons.Warnings.Warning):
+        class ICLTopDirWarning(SCons.Warnings.SConsWarning):
             pass
         if (is_mac or is_linux) and not env.Detect('icc') or \
            is_windows and not env.Detect('icl'):
@@ -495,15 +499,15 @@ def generate(env, version=None, abi=None, topdir=None, verbose=0):
                    'LIB'             : libdir,
                    'PATH'            : bindir,
                    'LD_LIBRARY_PATH' : libdir}
-            for p in list(paths.keys()):
-                env.PrependENVPath(p, os.path.join(topdir, paths[p]))
+            for p, v in paths.items():
+                env.PrependENVPath(p, os.path.join(topdir, v))
         if is_mac:
             paths={'INCLUDE'         : 'include',
                    'LIB'             : libdir,
                    'PATH'            : bindir,
                    'LD_LIBRARY_PATH' : libdir}
-            for p in list(paths.keys()):
-                env.PrependENVPath(p, os.path.join(topdir, paths[p]))
+            for p, v in paths.items():
+                env.PrependENVPath(p, os.path.join(topdir, v))
         if is_windows:
             #       env key    reg valname   default subdir of top
             paths=(('INCLUDE', 'IncludeDir', 'Include'),
@@ -569,7 +573,7 @@ def generate(env, version=None, abi=None, topdir=None, verbose=0):
         if not licdir:
             licdir = defaultlicdir
             if not os.path.exists(licdir):
-                class ICLLicenseDirWarning(SCons.Warnings.Warning):
+                class ICLLicenseDirWarning(SCons.Warnings.SConsWarning):
                     pass
                 SCons.Warnings.enableWarningClass(ICLLicenseDirWarning)
                 SCons.Warnings.warn(ICLLicenseDirWarning,
