@@ -9,7 +9,7 @@ selection method.
 """
 
 #
-# Copyright (c) 2001 - 2019 The SCons Foundation
+# __COPYRIGHT__
 #
 # Permission is hereby granted, free of charge, to any person obtaining
 # a copy of this software and associated documentation files (the
@@ -31,7 +31,7 @@ selection method.
 # WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #
 
-__revision__ = "/src/script/src/engine/SCons/Tool/jar.py none 2019-12-17 02:06:27 none"
+__revision__ = "__FILE__ __REVISION__ __DATE__ __DEVELOPER__"
 import os
 
 import SCons.Subst
@@ -101,7 +101,7 @@ def Jar(env, target = None, source = [], *args, **kw):
     # no target and want implicit target to be made and the arg
     # was actaully the list of sources
     if SCons.Util.is_List(target) and source == []:
-        SCons.Warnings.Warning("Making implicit target jar file, " +
+        SCons.Warnings.SConsWarning("Making implicit target jar file, " +
                               "and treating the list as sources")
         source = target
         target = None
@@ -123,7 +123,7 @@ def Jar(env, target = None, source = [], *args, **kw):
             target = os.path.splitext(str(source[0]))[0] + env.subst('$JARSUFFIX')
         except:
             # something strange is happening but attempt anyways
-            SCons.Warnings.Warning("Could not make implicit target from sources, using directory")
+            SCons.Warnings.SConsWarning("Could not make implicit target from sources, using directory")
             target = os.path.basename(str(env.Dir('.'))) + env.subst('$JARSUFFIX')
 
     # make lists out of our target and sources
@@ -158,7 +158,7 @@ def Jar(env, target = None, source = [], *args, **kw):
     #       files.
     def dir_to_class(s):
         dir_targets = env.JavaClassDir(source = s, *args, **kw)
-        if(dir_targets == []):
+        if dir_targets == []:
             # no classes files could be built from the source dir
             # so pass the dir as is.
             return [env.fs.Dir(s)]
@@ -192,7 +192,7 @@ def Jar(env, target = None, source = [], *args, **kw):
             except:
                 pass
 
-            SCons.Warnings.Warning("File: " + str(s) + " could not be identified as File or Directory, skipping.")
+            SCons.Warnings.SConsWarning("File: " + str(s) + " could not be identified as File or Directory, skipping.")
 
     # at this point all our sources have been converted to classes or directories of class
     # so pass it to the Jar builder
